package de.javamagazin.store;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.security.GeneralSecurityException;
import java.security.Key;
import java.security.KeyStore;
import java.security.Provider;
import java.security.SecureRandom;
import java.security.Security;

import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESedeKeySpec;

import org.apache.axis.components.logger.LogFactory;
import org.apache.axis.handlers.BasicHandler;
import org.apache.commons.logging.Log;

import com.ibm.crypto.provider.IBMJCE;

public abstract class AbstractSecurityHandler extends BasicHandler {
	protected KeyStore keyStore = null;
	protected Log log = LogFactory.getLog(getClass().getName());

	protected void loadKeyStore(String filename, char[] keyStorePasswd) {
			if ((filename == null) || (filename.equals(""))) {
				log.error("No keystore configured for security handler!");
				return;
			}
			try {
				loadKeyStore( new FileInputStream(filename), keyStorePasswd );
			} catch (FileNotFoundException e) {
				log.error("Exception while initializing security handler: " + e.getMessage());
			}
	}
	
	/** 
	 * shows (logs) the providers.
	 * @param p Provider list to show.
	 */
	protected void showProviders( Provider[] p ) {
		int i = 0;
		while (i < p.length) {
			log.info(Integer.toString(i) + ": " + p[i].getName() + " INFO: " + p[i].getInfo());
			i++;
		}
	}
	
	/**
	 * registers the new Security Provider dynamically.
	 *
	 */
	protected void registerSecurityProvider() {
		
		// just if you want to see, what is registered
		//showProviders( Security.getProviders() );

		log.info("Register the IBM Crypto Provider ...");

		Security.insertProviderAt(new IBMJCE(), 2);
		Security.removeProvider("SunJCE");

		// just if you want to see, what is registered
		//showProviders( Security.getProviders() );

	}
	protected void loadKeyStore( InputStream is, char[] keyStorePasswd ) {
		try {
			keyStore = KeyStore.getInstance("JCEKS");
			keyStore.load(is, keyStorePasswd);
			log.info("ClientSecurityHandler: Keystore successfully loaded.");		
		} catch( Exception e) {
			log.error("Exception while initializing security handler: " + e.getMessage());
			keyStore = null;
		}
	}

	protected Key createTripleDESKey() throws GeneralSecurityException {
		// generate random bytes
		byte[] rb = new byte[24];
		SecureRandom.getInstance("SHA1PRNG").nextBytes(rb);

		// create TripleDES key
		DESedeKeySpec keySpec = new DESedeKeySpec(rb);
		Key myKey = SecretKeyFactory.getInstance("DESede").generateSecret(keySpec);
		return myKey;
	}
}
