package de.javamagazin.store;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.security.Key;
import java.security.cert.X509Certificate;

import javax.xml.transform.stream.StreamSource;

import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.utils.XMLUtils;
import org.w3c.dom.Document;

import com.verisign.messaging.WSSecurity;
import com.verisign.xmlenc.AlgorithmType;
import com.verisign.xmlsig.KeyInfo;
import com.verisign.xmlsig.SigningKey;
import com.verisign.xmlsig.SigningKeyFactory;

public class ClientSecurityHandler extends AbstractSecurityHandler {
	public void init() {
		log.info("ClientSecurityHandler: Initialiazing...");
		
		InputStream keyStore = null;
		String keyStoreName = (String)getOption("keystore");

		registerSecurityProvider();

		try {
			keyStore = getClass().getResource( keyStoreName ).openStream();
		} catch (IOException e) {
			log.info("Exception while initialising ClientSecurityHandler: " + e.getMessage());
		}
		
		loadKeyStore(keyStore, "storePwd".toCharArray());

	}
	
	
	public void invoke(MessageContext msgCtx) throws AxisFault {
		if (keyStore == null)
			throw new AxisFault("ClientSecurityHandler: key store not initialized");
		try {
			Message msg = msgCtx.getRequestMessage();
			Document doc = msg.getSOAPEnvelope().getAsDocument();

			// uncomment to see SOAP message
			//String str = XMLUtils.DocumentToString(doc);
			//log.info(str);

			signAndEncrypt(doc);

			String encStr = XMLUtils.DocumentToString(doc);
			StreamSource s = new StreamSource(new StringReader(encStr));

			// uncomment to see encrypted SOAP message
			//log.info(encStr);

			msg.getSOAPPart().setContent(s);

		} catch (Exception e) {
			// added sr for debugging
			e.printStackTrace();
			throw new AxisFault("Exception in ClientSecurityHandler: " + e.getMessage());
		}
	}
	
	private void signAndEncrypt(Document soapMsg) throws Exception {
		log.info("ClientSecurityHandler: Encrypting and signing the SOAP request...");
		//	   use client's private key as the signing key
		Key privateKeyClient = keyStore.getKey("Client", "keyPwd".toCharArray());
		SigningKey signingKey = SigningKeyFactory.makeSigningKey(privateKeyClient);

		//	   create key info for the signing key
		KeyInfo signingKeyInfo = new KeyInfo();
		signingKeyInfo.setCertificate((X509Certificate)keyStore.getCertificate("Client"));

		//	   create a new symmetric data encryption key
		Key dataEncKey = createTripleDESKey();

		//	   get server's public key for encryption of the
		//	   symmetric data encryption key
		Key keyEncKey = keyStore.getCertificate("Server").getPublicKey();
		KeyInfo encryptingKeyInfo = null;

		WSSecurity wsse = new WSSecurity();
		wsse.signAndEncrypt(
			soapMsg,
			signingKey,
			signingKeyInfo,
			dataEncKey,
			AlgorithmType.TRIPLEDES,
			keyEncKey,
			AlgorithmType.RSA1_5,
			encryptingKeyInfo);
		log.info("ClientSecurityHandler: Done.");
	}
}
